@@ -24,6 +24,7 @@ from concurrent.futures import ThreadPoolExecutor
 from raytils.datasets.coco_fields import CocoCategories, CocoInfo, CocoLicence, CocoDetectionAnnotation, CocoModes, \
     CocoList, CocoImage, combine_coco_objects
 from raytils.perception import get_image_size
+from raytils.system.download import Downloader
 
 
 class Annotation:
@@ -237,6 +238,9 @@ class COCOCreator:
             coco_data = json.load(fh)
 
         image_id_to_filename = {c["id"]: c["file_name"] for c in coco_data["images"]}
+        for file_name in image_id_to_filename.values():
+            self.add_detection_image(file_name)  # add empty file_paths
+
         category_id_to_class_name = {c["id"]: c["name"] for c in coco_data["categories"]}
         for annotation in coco_data["annotations"]:
             bbox = annotation.get("bbox")
@@ -252,23 +256,42 @@ class COCOCreator:
         with json_path.open("rb") as fh:
             darwin = json.load(fh)
 
-        file_id_to_name = {str(idx): d for idx, d in enumerate(darwin["image"]["frame_urls"])}
+        # Download all the files and copy locally
+        file_downloader = Downloader(list(darwin["image"]["frame_urls"]))
+        file_url_to_local = file_downloader.get_all()
+        file_id_to_name = {str(idx): file_url_to_local[d] for idx, d in enumerate(darwin["image"]["frame_urls"])}
+
         for path in file_id_to_name.values():  # add all images in-case of no annotations for image
             self.add_detection_image(path)
 
+        def clamp(n, smallest, largest):
+            return max(smallest, min(n, largest))
         annotations = {}  # frame_id: [annotations]
-        for ann in darwin["annotations"]:
+        for instance_id, ann in enumerate(darwin["annotations"]):
             class_name = ann["name"]
             interpolated = ann["interpolated"]
+            # Darwin doesn't actually create instance_ids for each of its tracks so assume them
             # validate all instance_ids are consistent
-            ids = set([ii["instance_id"]["value"] for ii in ann["frames"].values() if "instance_id" in ii])
-            if not ids:
-                continue
-            if len(ids) > 1:
-                raise ValueError("Instance IDS not unique")
-            instance_id = ids.pop()
+            # ids = set([ii["instance_id"]["value"] for ii in ann["frames"].values()])
+            # if not ids:
+            #     continue
+            # if len(ids) > 1:
+            #     raise ValueError("Instance IDS not unique")
+            # instance_id = ids.pop()
             for frame_id, frame_annotation in ann["frames"].items():
                 bbox = [frame_annotation["bounding_box"].get(a) for a in ["x", "y", "w", "h"]]
+                # Clamp values to frame
+                bbox[0] = clamp(bbox[0], 0, darwin["image"]["width"])
+                bbox[1] = clamp(bbox[1], 0, darwin["image"]["height"])
+                if bbox[2] + bbox[0] >= darwin["image"]["width"]:
+                    bbox[2] -= darwin["image"]["width"] - (bbox[2] + bbox[0])
+                if bbox[3] + bbox[1] >= darwin["image"]["height"]:
+                    bbox[3] -= darwin["image"]["height"] - (bbox[3] + bbox[1])
+                # ensure all are positive
+                bbox = [abs(x) for x in bbox]
+                if any([x < 1 for x in bbox[2:]]):
+                    print("CHECK THIS")
+                    print(bbox)
                 if frame_id not in annotations:
                     annotations[frame_id] = []
                 self.add_detection_annotation(image_path=file_id_to_name[frame_id], bbox=bbox, class_name=class_name,
@@ -402,6 +425,7 @@ class COCOCreator:
         images_not_local = False
         tmp_dir = f"/tmp/coco_creator/{uuid.uuid4()}"
 
+        # Check if any files need to be downloaded
         # Write image data to new packaged dataset
         for image_file, image_annotation in image_path_lut.items():
             original_file = image_file
