@@ -33,7 +33,8 @@ class Metrics:
     @staticmethod
     def _is_better(metric_name, val1, val2):
         higher_better = ["explained_variance_score", "r2_score", ]
-        lower_better = ["max_error", "mean_absolute_error", "mean_absolute_percentage_error", "mean_gamma_deviance",
+        lower_better = ["max_error", "l1_loss", "l2_loss",
+                        "mean_absolute_error", "mean_absolute_percentage_error", "mean_gamma_deviance",
                         "mean_poisson_deviance", "mean_squared_error", "mean_squared_log_error",
                         "mean_squared_percentage_error", "mean_tweedie_deviance", "median_absolute_error", "min_error",
                         "relative_absolute_error", "relative_squared_error", "residual_sum_of_squares",
@@ -59,6 +60,14 @@ class Metrics:
             _computed_metrics[metric_name] = metric_value
         return _computed_metrics
 
+    def l1_loss(self, y_true, y_pred):
+        l1 = np.sum(np.abs(y_true - y_pred))
+        return l1
+
+    def l2_loss(self, y_true, y_pred):
+        l2 = np.sum(np.square(y_true - y_pred))
+        return l2
+
     def residual_sum_of_squares(self, y_true, y_pred):
         rss = np.sum(np.square(y_true - y_pred))
         return rss
